from django.db import models
from django.core.exceptions import ValidationError
from django.urls import reverse
from django.utils.text import slugify

from django_prose_editor.fields import ProseEditorField


def validate_image_size(value):
    filesize = value.size
    if filesize > 1 * 1024 * 1024:  # 1MB
        raise ValidationError("Maximum file size is 1MB")


class Currency(models.Model):
    code = models.CharField(max_length=3, unique=True)  # ISO 4217 currency code
    name = models.CharField(max_length=50)
    symbol = models.CharField(max_length=5)

    class Meta:
        verbose_name_plural = "Currencies"
        ordering = ["code"]

    def __str__(self):
        return f"{self.code} ({self.name})"


class Plan(models.Model):
    name = models.CharField(max_length=100)
    description = ProseEditorField()
    service = models.ForeignKey(
        "Service", on_delete=models.CASCADE, related_name="plans"
    )
    is_default = models.BooleanField(default=False)
    features = ProseEditorField(blank=True)
    order = models.IntegerField(default=0)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ["order", "name"]
        unique_together = [["service", "name"]]

    def __str__(self):
        return f"{self.service.name} - {self.name}"

    def save(self, *args, **kwargs):
        if self.is_default:
            # Ensure only one default plan per service
            Plan.objects.filter(service=self.service).update(is_default=False)
        super().save(*args, **kwargs)

    def clean(self):
        super().clean()
        # If this is the only plan, make it default
        if self.pk and self.service and self.service.plans.count() == 1:
            self.is_default = True


class PlanPrice(models.Model):
    plan = models.ForeignKey(Plan, on_delete=models.CASCADE, related_name="prices")
    currency = models.ForeignKey(Currency, on_delete=models.PROTECT)
    price = models.DecimalField(max_digits=10, decimal_places=2)

    class Meta:
        unique_together = [["plan", "currency"]]

    def __str__(self):
        return f"{self.plan.name} - {self.currency.code} {self.price}"


class Category(models.Model):
    name = models.CharField(max_length=100)
    slug = models.SlugField(unique=True)
    parent = models.ForeignKey(
        "self", on_delete=models.CASCADE, null=True, blank=True, related_name="children"
    )
    description = models.TextField(blank=True)
    order = models.IntegerField(default=0)

    class Meta:
        verbose_name_plural = "Categories"
        ordering = ["order", "name"]

    def __str__(self):
        if self.parent:
            return f"{self.parent} > {self.name}"
        return self.name

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name)
        super().save(*args, **kwargs)

    @property
    def full_path(self):
        path = [self.name]
        parent = self.parent
        while parent:
            path.append(parent.name)
            parent = parent.parent
        return " > ".join(reversed(path))


class CloudProvider(models.Model):
    name = models.CharField(max_length=100)
    slug = models.SlugField(unique=True)
    description = ProseEditorField(blank=True)
    logo = models.ImageField(
        upload_to="cloud_provider_logos/",
        validators=[validate_image_size],
        null=True,
        blank=True,
    )

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name)
        super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse("services:provider_detail", kwargs={"slug": self.slug})


class ConsultingPartner(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField(unique=True)
    description = ProseEditorField(blank=True)
    logo = models.ImageField(
        upload_to="partner_logos/",
        validators=[validate_image_size],
        null=True,
        blank=True,
    )
    website = models.URLField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name)
        super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse("services:partner_detail", kwargs={"slug": self.slug})


class Service(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField(max_length=250, unique=True)
    description = ProseEditorField()
    cloud_provider = models.ForeignKey(CloudProvider, on_delete=models.CASCADE)
    consulting_partners = models.ManyToManyField(
        ConsultingPartner, related_name="services", blank=True
    )
    categories = models.ManyToManyField(Category, related_name="services")
    features = ProseEditorField()
    logo = models.ImageField(
        upload_to="service_logos/",
        validators=[validate_image_size],
        null=True,
        blank=True,
    )
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.slug:
            base_slug = f"{self.name}-{self.cloud_provider.name}"
            self.slug = slugify(base_slug)

            # If slug exists, append number
            counter = 1
            while Service.objects.filter(slug=self.slug).exists():
                self.slug = f"{slugify(base_slug)}-{counter}"
                counter += 1

        super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse("services:service_detail", kwargs={"slug": self.slug})

    def get_default_plan(self):
        return self.plans.filter(is_default=True).first() or self.plans.first()


class Lead(models.Model):
    service = models.ForeignKey(Service, on_delete=models.CASCADE)
    name = models.CharField(max_length=200)
    company = models.CharField(max_length=200)
    email = models.EmailField()
    phone = models.CharField(max_length=50)
    created_at = models.DateTimeField(auto_now_add=True)
    odoo_lead_id = models.IntegerField(null=True, blank=True)

    def __str__(self):
        return f"{self.name} - {self.company} ({self.service})"
